function [E, a, H] = Boson1D(lambda, N, neig, lancz)
% Build and diagonalize the Hamiltonian for a one-dim oscillator
% with a simple perturbation.
% Creation-annihilation operators are built via Kronecker products.
% V contains the perturbation: default V = a^2 + h.c.
% (a' in matlab is the Hermitian conjugate).
%
% Usage: 
%          [E, a, H] = Boson1D(lambda, N, neig, lancz)
%
% lambda is the coupling in H=H0+lambda*V
% V should be defined in the source code.
% N is the maximum occupation number in each oscillator
% neig is the number of eigenvalues to be computed
% lancz is the number of Lanczos vectors to be used by the Arnoldi algorithm
% 
% Copyright ©2013-2015-2020 by E. Onofri

if nargin<4, lancz = 32; end;
if nargin<3, neig = 16; end;
if nargin<2, N = 32; end;
if nargin<1, lambda = 0.1; end;

a = boson(N);

opts.tol    = 1e-10; % to be used by eigs 
opts.disp   = 0;    % eigs works quietly - alternatively =1 , =2.
opts.isreal = true;
opts.p      = lancz ;
opts.disp = 1;

H0 = a'*a; % unperturbed Hamiltonian

q = (a+a')/sqrt(2);      
V = q^4;                                % <-------- define V here

H = H0 + lambda * V;

clear H0, V;                       % save memory

E = eigs(H, neig, 'SA', opts);
stem(E,'fill')
grid on

end

%%-------------------------- Creation/annihilation operators--------
function a = boson(B)
D  = (0:B)';                   % diagonal mat.elem. (column vector)
a = spdiags(sqrt(D), 1, B, B); % annihilation operator
whos
end
